<!---  these UDF's are part of the Techspedition core file and covered by its license agreement --->
<cfscript>
function NextInQ (aFuseaction) {
	/*
	 * insert a fuseaction at the front of the localQ (requires Fusebox core file from Techspedition.com)
	 * 
	 * @param aFuseaction 	 The fully qualified fuseaction to insert at the front of the localFuseQ
	 * @return returns TRUE when done
	 * @author John Quarto-vonTivadar (johnq@techspedition.com) 
	 * @version 1.01, April 30, 2002 
	 */
	var CCVmode = "";
	if (techspedition.ContentComponentVariable NEQ "") CCVmode = " INTO " & techspedition.ContentComponentVariable & " APPEND";
	techspedition.localQ = ListPrepend(techspedition.localQ,aFuseaction & CCVmode);
	return true;
}
function BackOfQ (aFuseaction) {
	/*
	 * insert a fuseaction at the end of the FuseQ (requires Fusebox core file from Techspedition.com)
	 * 
	 * @param aFuseaction 	 The fully qualified fuseaction to insert at the end of the FuseQ
	 * @return returns TRUE when done
	 * @author John Quarto-vonTivadar (johnq@techspedition.com) 
	 * @version 1.01, January 15, 2002 
	 */
	techspedition.FuseQ = ListAppend(techspedition.FuseQ,aFuseaction);
	return true;
}
function FrontOfQ (aFuseaction) {
	/*
	 * insert a fuseaction to the front of the FuseQ (requires Fusebox core file from Techspedition.com)
	 * 
	 * @param aFuseaction 	 The fully qualified fuseaction to insert to the front of the FuseQ
	 * @return returns TRUE when done
	 * @author John Quarto-vonTivadar (johnq@techspedition.com) 
	 * @version 1.01, January 15, 2002 
	 */
	techspedition.FuseQ = ListPrepend(techspedition.FuseQ,aFuseaction);
	return true;
}
function AddToQ (aFuseaction) {
	/*
	 * insert a fuseaction at the end of the localQ (requires Fusebox core file from Techspedition.com)
	 * 
	 * @param anExtendedFuseaction 	 The fully qualified fuseaction to insert at the end of the localFuseQ
	 * @return returns TRUE when done
	 * @author John Quarto-vonTivadar (johnq@techspedition.com) 
	 * @version 1.01, April 30, 2002 
	 */
	var CCVmode = "";
	
	if (techspedition.ContentComponentVariable NEQ "") CCVmode = " INTO " & techspedition.ContentComponentVariable & " APPEND";
	techspedition.localQ = ListAppend(techspedition.localQ,aFuseaction & CCVmode);
	return true;
}
function ClearQ () {
	/*
	 * clears the FuseQ and the localQ by setting the value to the empty string (requires Fusebox core file from Techspedition.com)
	 * 
	 * @param aQueue, OPTIONAL, if aQ is specified then it clears only that queue by setting its value to the empty string, valid values are "localQ" and "FuseQ"
	 * 		  If no parameter is passed in then both techspedition.FuseQ and techspedition.localQ are cleared
	 * @return returns TRUE if the queue was modified, FALSE if not
	 * @author John Quarto-vonTivadar (johnq@techspedition.com) 
	 * @version 1.01, April 28, 2002 
	 */

	var aQueue = "";
	var flag = FALSE;
	
	if (ArrayLen(Arguments) gt 0) aQueue = Arguments[1];
	
	if ((aQueue EQ "FuseQ") OR (aQueue EQ "techspedition.FuseQ")) {
		techspedition.FuseQ = "";
		flag = TRUE;
	}
	
	if ((aQueue EQ "localQ") OR (aQueue EQ "techspedition.localQ")) {
		techspedition.localQ = "";
		flag = TRUE;
	}
	
	if (aQueue EQ "") {
		techspedition.FuseQ = "";
		techspedition.localQ = "";
		flag = TRUE;
	}
	
	return flag;
}

function SuperCircuit () {
	/*
	 * given a circuit, determine the circuit's parent circuit. If it has no parent (i.e., the circuit is the top-most circuit as defined by the fbx_circuits file for this app) then return "" (requires Fusebox core file from Techspedition.com)
	 * 
	 * @param aCircuit, OPTIONAL, the circuit alias whose parent circuit (the "super") is to be determined. 
	 * 		  If no circuit is passed in as a parameter, then the current circuit is assumed
	 * @return returns the "super", the circuit alias of the parent
	 * @author John Quarto-vonTivadar (johnq@techspedition.com) 
	 * @version 1.01, January 15, 2002 
	 */
	var aCircuit = fusebox.thiscircuit;
	var thisSuper = "";
	
	if (ArrayLen(Arguments) gt 0) aCircuit = Arguments[1];
	
	if (StructKeyExists( techspedition_.Circuits.VirtualPath, aCircuit ) ) {
		if (ListLen(techspedition_.Circuits.VirtualPath[aCircuit]) GT 1) {
			thisSuper = ListGetAt(techspedition_.Circuits.VirtualPath[aCircuit],ListLen(techspedition_.Circuits.VirtualPath[aCircuit])-1);
		}
	}

	return thisSuper;
}
function Super() {
	/*
	* this function is reserved for future use
	*/
	return true;
}
function SuperQ () {
	/*
	 * "throws" a fuseaction up to the same fuseaction in its parent circuit. Uses AddToQ() to prepend a fuseaction 
	 * to the techspedition.FuseQ , with the fuseaction having its current circuit stripped out and the current circuit's parent circuit substituted in its place
	 * by using "supercircuit()"  
	 * If the fuseaction's circuit is the top-most then no action is performed (else an infinite loop would occur). 
	 * (requires Fusebox core file from Techspedition.com)
	 * 
	 * @param aFuseaction, OPTIONAL, a fully qualified Fuseaction
	 *        If no Fuseaction is passed in, then the current circuit and current fuseaction are assumed
	 * @return returns TRUE if the techspedition.localQ was modified, FALSE if not
	 * @author John Quarto-vonTivadar (johnq@techspedition.com) 
	 * @version 1.0, January 15, 2002 
	 */
	var aCircuit    = SuperCircuit(fusebox.thiscircuit);
 	var aFuseaction = fusebox.fuseaction;
	var mode = techspedition.FuseMode;
	var x = false;
	
	if (ArrayLen(Arguments) gt 0) {
		aCircuit    = SuperCircuit(ListGetAt( Arguments[1], 1,  '.' ));
	 	aFuseaction = ListGetAt( Arguments[1], 2,  '.' );
	}
			
	if (aCircuit EQ "") {
		return false;
	
	} else {
		x = AddToQ("#aCircuit#.#aFuseaction# AS #mode#");
		return true;
	}
}

function IsHomeCircuit () {
	/*
	 * this UDF mimics the functionality of the Fusebox API variable IsHomeCircuit
	 * 
	 * @param aCircuit, OPTIONAL, a fully qualified Fuseaction
	 *        If no Circuit is passed in, then the current circuit is assumed
	 * @return returns TRUE if the current circuit is the top-most "Home" circuit, FALSE if not
	 * @author John Quarto-vonTivadar (johnq@techspedition.com) 
	 * @version 1.0, January 15, 2002 
	 */
	var aCircuit = fusebox.thiscircuit;
	if (ArrayLen(Arguments) gt 0) aCircuit = Arguments[1];
	
	if (aCircuit EQ fusebox.homecircuit)
		return true;
	else
		return false;
}
function IsTargetCircuit () {
	/*
	 * this UDF mimics the functionality of the Fusebox API variable IsTargetCircuit
	 * 
	 * @param aCircuit, OPTIONAL, a fully qualified Fuseaction
	 *        If no Circuit is passed in, then the current circuit is assumed
	 * @return returns TRUE if the current circuit is the target circuit, FALSE if not
	 * @author John Quarto-vonTivadar (johnq@techspedition.com) 
	 * @version 1.0, January 15, 2002 
	 */
	var aCircuit = fusebox.thiscircuit;
	if (ArrayLen(Arguments) gt 0) aCircuit = Arguments[1];
	
	if (aCircuit EQ fusebox.targetcircuit)
		return true;
	else
		return false;
}
function FlushContent () {
	/*
	 * this UDF flushes any working content from techspedition.WorkingContent to techspedition.GeneratedContent, thus making such content "permanent" for this page request and wiping the WorkingContent variable clean. Note that the flush will occur when the current fuseaction being executed is complete and that techspedition.FlushContent is set to FALSE at the start of every fuseaction's execution
	 * 
	 * @return returns TRUE if the variable techspedition.FlushContent is succesfully set to TRUE
	 * @author John Quarto-vonTivadar (johnq@techspedition.com) 
	 * @version 1.0, April 29, 2002 
	 */
	techspedition.FlushContent = TRUE;
	return true;
}
function SuppressContent () {
	/*
	 * this UDF suppresses any content generated from the current fuseaction from being displayed. This will not affect content being pushed INTO a variable. Note that techspedition.SuppressContent is set to FALSE at the start of every fuseaction's execution, and is the current fuseaction is being pushed INTO a varaible it is set to TRUE automatically.
	 * 
	 * @return returns TRUE if the variable techspedition.SuppressContent is succesfully set to TRUE
	 * @author John Quarto-vonTivadar (johnq@techspedition.com) 
	 * @version 1.0, April 29, 2002 
	 */
	techspedition.SuppressContent = TRUE;
	return true;
}
	
function isPermitted( clientroles ){
	/*
	 * this UDF returns true if user is allowed based on Security.cfm in home circuit. 
	 * 
	 * @author Hal Helms (hal@techspedition.com) 
	 * @version 2.0, May 15, 2002 
	 */
	var circuitAccessPermitted = true;

	if( StructKeyExists( Security[fusebox.thisCircuit], 'circuitAccess' ) ) {
		circuitAccessPermitted = false;
		for(i=1;i LTE ListLen( clientroles );i=i+1 ){
			if( ListFindNoCase( ReplaceNoCase( Security[fusebox.thisCircuit]['circuitaccess'],', ', ',', 'ALL' ), Trim( ListGetAt( clientRoles, i )))) {
				circuitAccessPermitted = true;
				break;
			}
		}
	}
	if( circuitAccessPermitted is false) return false;
	
	if( NOT StructKeyExists( Security[fusebox.thisCircuit], fusebox.fuseaction ) )return true;
		for(i=1;i LTE ListLen( clientRoles );i=i+1 ){
	  		if( ListFindNoCase( ReplaceNoCase( Security[fusebox.thisCircuit][fusebox.fuseaction], ', ',',', 'All' ), Trim( ListGetAt( clientRoles, i )))) return TRUE;
		}
	return FALSE;
}
</cfscript>
